<?php

/*
PHP class to parse and decode a bencoded file. Used by torrent files.
Copyright (C) 2006, 2007  Vegard Hammerseth <thewulong@gmail.com> (http://thewulong.com)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

v1.0.2
*/

class bcode
{

	var $keys = array(
							"length",
							"name",
							"piece length",
							"announce",
							"comment",
							"created by",
							"creation date",
							"encoding"
							);

	function decode($arg_str)
	{
		$str 		= bcode::get_fixed_data($arg_str);
		$array 	= bcode::parse_to_array($str);
		if (count($array) < 1)
		{
			return;
		}
		
		$array 	= bcode::sort_parsed_array($array);
		if (count($array) < 1)
		{
			return;
		}	
		
		return $array;
	}
	
	
	
	/* remove unwanted pieces and change some formats */
	function get_fixed_data($arg_str)
	{
		/* remove the unwanted pieces */
		$pos	= strpos($arg_str,"pieces")+30;
		$str	= substr($arg_str,0,$pos);
		$str	= preg_replace_callback("/i(\d+)e(\d+)\:/",create_function('$m','return strlen($m[1]).":".$m[1].$m[2].":";'),$str);
		$str	= preg_replace("/i\dee(\d+)\:/","\\1:",$str);
		
		return $str;
	}	
	
	
	
	/* this is the function who does the work, from bcode to array */
	function parse_to_array($arg_str)
	{
		$str 		= $arg_str;
		$return 	= array();
		$length 	= 1;

		/* run through the string and get chunks of it */		
		while (strlen($str) > 1)
		{
			$info = substr($str,0,$length);
			$str	= substr($str,$length);
			$return[] = $info;
			
			/* here we find how long the next str should be */
			$length = "";
			while (strlen($str) > 1)
			{
				$chr	= substr($str,0,1);
				$str	= substr($str,1);
				/* we got our length number */
				if ($chr == ":")
				{
					break;
				}
				/* found unwanted letter, jump to next */
				if (!is_numeric($chr))
				{
					continue;
				}
				$length .= $chr;
			}
		}
		
		return array_slice($return,0,-2);
	}
	
	
	
	/* here we turn our parsed array into a humanish format */
	function sort_parsed_array($arg_array = array())
	{
		if (!is_array($arg_array))
		{
			return;
		}
		
		/* to support PHP4 */
		$bcode = new bcode;
		$keys = $bcode->keys;
		
		$array	= $arg_array;
		$return 	= array();
		$path 	= $continue = "";
		
		for ($x=0;$x<count($array);$x++)
		{
			/* set regular values, exept length */
			if ($array[$x] != "length" && in_array($array[$x],$keys))
			{
				$return[$array[$x]] = $array[$x+1];
				$x++;
				continue;
			}

			/* start file section */
			if ($array[$x] == "files")
			{
				$files = 1;
				continue;
			}
	
			/* file section is started */
			if (isset($files))
			{
				/* let's put the file info in our array */
				if ($array[$x] == "length")
				{
					if (isset($path,$length))
					{
						$return['files'][] = array("path"=>substr($path,0,-1),"size"=>$length);
					}
					$path = $continue = "";
					$x++;
					$length = $array[$x];
					continue;
				}
				
				/* we don't want "path" in our $path, continue to next line */
				if ($array[$x] == "path" || !empty($continue))
				{
					continue;
				}
				/* some torrent makers add path and path.utf-8, let's skip path.utf-8 so we don't get double */
				elseif (isset($return['encoding']) && strtolower($array[$x]) == "path.".strtolower($return['encoding']))
				{
					$continue = 1;
					continue;
				}
				
				/* add to path */
				$path .= $array[$x]."/";
			}
			
			/* set regular values, length */
			if (in_array("length",$keys) && $array[$x] == "length")
			{
				$x++;
				$return['length'] = $array[$x];
			}
		}
		/* let's add the last path */
		if (!empty($path) && !empty($length))
		{
			$return['files'][] = array("path"=>substr($path,0,-1),"size"=>$length);
		}
		
		return $return;
	}
}

?>