<?php

/*
PHP class with a collection of usefull functions
Copyright (C) 2005, 2007  Vegard Hammerseth <thewulong@gmail.com> (http://thewulong.com)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

v1.0.3
*/


class etc
{

	/* calculate byte into human format */
	function byte_change($arg_byte,$arg_decimals = "2",$arg_type = "")
	{
		if (!is_numeric($arg_byte))
		{
			return $arg_byte;
		}
		
		$units = array("B","KB","MB","GB","TB","PB","EB","ZB","YB");
		
		for ($x=count($units)-1;$x>=0;$x--)
		{
			$nr = pow(1024,$x);
			if ($arg_byte >= $nr || strtoupper($arg_type) == $units[$x])
			{
				$x--;
				break;
			}
		}
		$x++;

		$return = @round($arg_byte/$nr,$arg_decimals);
		
		/* we already know unit so we don't need it */
		if (empty($arg_type) && $x >= 0)
		{
			$return .= " ".$units[$x];
		}
		
		return $return;
	} // 1.0.3

	

	/* show time left in a human format */
	function human_time_left($arg_time,$arg_length = "short",$arg_last_unit = "sec")
	{
		$return = "";
		$length = ($arg_length == "long" || $arg_length == "long"?$arg_length:"short");
		$time   = $arg_time;
		$names['long']  = array("year","day","hour","minute","second");
		$names['short'] = array("yr","day","hr","min","sec");
		$times = array(31536000,86400,3600,60,1);
		
		/* loop through all times */
		for ($x=0;$x<sizeof($times);$x++)
		{
			$word = $names[$length][$x];
			$out=0;
			
			/* should we stop unit here? */
			if ($arg_last_unit == $word)
			{
				break;
			}
			
			/* find out what time */
			while ($time >= $times[$x])
			{
				$time -= $times[$x];
				$out++;
			}
			
			if ($out > 0)
			{
				$return .= etc::number_word($out,$word,$word."s").", ";
			}
		}
		
		return substr($return,0,-2);
	} // 1.0.0



 	/* function to print "1 hour" and not "1 hours" */
	function number_word($arg_return,$arg_word,$arg_words)
	{
		if ($arg_return == "1") {
			$arg_return .= " ".$arg_word;
		}
		else
		{
			$arg_return .= " ".$arg_words;
		}
		
		return $arg_return;
	} // 1.0.0



	/* "enable" the file_put_contents in PHP4 */
	function php4_file_put_contents($arg_file,$arg_data)
	{
		if (!$fsock = @fopen($arg_file,"w"))
		{
			return;
		}
		if (!fwrite($fsock,$arg_data))
		{
			return;
		}
		fclose($fsock);
		return true;
	} // 1.0.0



	/* add '...' after words being to long */
	function subword($arg_string,$arg_length)
	{
		if (strlen($arg_string) <= $arg_length+3)
		{
			return $arg_string;
		}

		return substr($arg_string,0,$arg_length)."...";
	} // 1.0.0



	/* remove a file or directory */
	function rmrf($arg_path)
	{
		/* file don't exists */
		if (!file_exists($arg_path))
		{
			return;
		}

		/* we got a file */
		if (is_file($arg_path))
		{
			if (@unlink($arg_path))
			{
				return true;
			}
		}
		elseif (is_dir($arg_path) && $dopen = opendir($arg_path))
		{
			/* loop througth all files in $arg_path */
			while ($f = readdir($dopen))
			{
				if ($f == "." || $f == "..")
				{
					continue;
				}
				
				if (is_dir($arg_path."/".$f))
				{
					/* run function again, since we got a dir */
					etc::rmrf($arg_path."/".$f);
				}
				else
				{
					@unlink($arg_path."/".$f);
				}
			}
			fclose($dopen);
			
			/* finally, remove desired path */
			if (@rmdir($arg_path))
			{
				return true;
			}
		}
	} // 1.0.0



	/* find executable path for program */
	function exec_path($arg_cmd)
	{
   	$paths = array("/usr/bin","/bin","/usr/local/bin","/usr/sbin","/sbin");
		$paths = array_merge($paths,explode(":",$_SERVER['PATH']));
		$paths = array_unique($paths);


		/* find and execute path */
		foreach ($paths as $path)
		{
			if (is_executable($path."/".$arg_cmd))
			{
				return $path."/".$arg_cmd;
			}
		}
	} // 1.0.0


	
	/* be able to execute files even with safe_exec on! */
	function safe_exec($arg_cmd)
	{
		$cmdlist = explode(" ",$arg_cmd);
	   $cmd     = $cmdlist['0'];
      $args    = implode(" ",array_slice($cmdlist,1));

		if (!is_executable($cmd) && !$path = etc::exec_path($cmd))
		{
			$popen = popen($arg_cmd,"r");
		}
		else
		{
			$popen = popen("$cmd $args","r");
		}
		
	   /* execute command with arguments and return result */
      if (!$popen)
      {
      	return;
      }
		
		$return = "";
		while (!feof($popen))
		{       
			$return .= fgets($popen,4096);
		}
		pclose($popen);
               
      return $return;
	} // 1.0.0



	/* remove an file extention, if any */
	function remove_ext($arg_string,$arg_ext = "")
	{
		if (empty($arg_ext))
		{
			return substr($arg_string,0,-strlen(strrchr($arg_string,".")));
		}
		elseif (substr($arg_string,-strlen(".$arg_ext")) == ".$arg_ext")
		{
			return substr($arg_string,0,-strlen(".$arg_ext"));
		}
		else
		{
			return $arg_string;
		}
	} // 1.0.0



	/* valid url */
	function valid_url($arg_url)
	{
		if (preg_match("/(http(s?)|ftp):\/\//i",$arg_url) && fopen($arg_url,"r"))
		{
			return true;
		}
	} // 1.0.0


	/* is path in web tree? */
	function in_webtree($arg_path)
	{
		$path = realpath($arg_path);
		if (!file_exists($path))
		{
			return false;
		}

		if ($_SERVER['DOCUMENT_ROOT'] == substr($path,0,strlen($_SERVER['DOCUMENT_ROOT'])))
		{
			return true;
		}
	} // 1.0.0
	
	
	/* is a dir/file removable */
	function is_removeable($arg_path) // this function used to have a purpose
	{
		return true;
		if (file_exists($arg_path) && (is_writeable($arg_path) || fileowner($arg_path) == posix_getuid()))
		{
			return true;
		}
	} // 1.0.0
	
	
	/* check if a file is in use */
	function in_use($arg_path)
	{
		if (file_exists($arg_path) && filemtime($arg_path) > time()-2)
		{
			return true;
		}
	} // 1.0.0
	
	
	/* does client use WAP protocol? */
	function is_wap()
	{
		if (isset($_SERVER['HTTP_X_WAP_PROFILE']) || preg_match("/(MIDP|MMP|WAP|MOBILE|SYMBIAN)/i",$_SERVER['HTTP_USER_AGENT']))
		{
			return true;
		}
	} // 1.0.0
}

?>
