<?php

/*
w3btorrent (keep it simple) - simple web fronted torrent client
Copyright (C) 2005, 2007  Vegard Hammerseth <thewulong@gmail.com> (http://thewulong.com)

This file is part of w3btorrent.

w3btorrent is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
any later version.
*/

require_once("inc/class/bcode.class.php");


class torrent
{
	

	/* show metainfo */
	function metainfo($arg_torrentfile)
	{
		if (!is_file($arg_torrentfile) || !is_readable($arg_torrentfile))
		{
			return;
		}
		$data = file_get_contents($arg_torrentfile); 
		$meta = bcode::decode($data);

		/* is this a valid torrent file? */
		if (isset($meta['announce'],$meta['name']))
		{
			return $meta;
		}
	}



	/* get size of torrent */
	function size($arg_torrentfile)
	{
		/* get meta info */
		$metainfo = torrent::metainfo($arg_torrentfile);

		$size = 0;
		if (isset($metainfo['length']))
		{
			$size = $metainfo['length'];
		}
		elseif (isset($metainfo['files']))
		{
			foreach ($metainfo['files'] as $value)
			{	
				$size += $value['size'];
			}
		}

		return $size;
	}



	/* do we got a valid torrent file? */
	function valid_file($arg_torrentfile)
	{
		if (torrent::metainfo($arg_torrentfile))
		{
			return true;
		}
	}



	/* fix status in a proper format */
	function status($arg_statusfile)
	{
		if (!is_file($arg_statusfile) || !is_readable($arg_statusfile))
		{
			return;
		}

		/* we don't want "\n" so we don't use file() */
		$status = explode("\n",file_get_contents($arg_statusfile));

		/* is it running? */
		if (isset($status['0'],$status['1']) && is_numeric($status['0'].$status['1']) && posix_getsid($status['0']) && ($status['1']+10) > time())
		{
			$return['pid'] = $status['0'];
			/* lights */
			if (($status['3']+$status['4']+$status['5']) < 1)
			{
				$return['led'] = 2;
			}
			elseif ($status['7'] != $status['8'])
			{
				$return['led'] = 3;
			}
			else
			{
				$return['led'] = 4;
			}
		}
		else
		{
			$return['pid'] = false;
			$return['led'] = 0;
		}
		$return['time']         = $status['1'];
		$return['path']			= trim($status['2']);
		
		$return['seeders']	= $status['3'];
		$return['leechers'] 	= $status['4'];
		$return['peers']		= $status['5'];		
		
		$return['pices_completed']		= $status['6'];
		$return['pices_available']		= $status['7'];
		$return['pices_in_torrent']	= $status['8'];

		/* size; downloaded x/y MB */
		$return['current_size'] = (is_numeric($status['9'])?$status['9']:0);
		$return['torrent_size'] = (is_numeric($status['11'])?$status['11']:0);
		$return['uploaded']		= (is_numeric($status['10'])?$status['11']:0);

		/* progress, percent */
		$return['percent']   = @floor(($return['current_size']/$return['torrent_size'])*100);

		/* up/down */
		$return['down']      = (is_numeric($status['12'])?$status['12']:0);
		$return['up']			= (is_numeric($status['13'])?$status['13']:0);
		
		/* let's find the average speed, since CTorrent is unrelayable */
		if ($return['pid'])
		{
			/* download speed */
			if (isset($_SESSION[$_SERVER['REMOTE_ADDR']][$arg_statusfile]['down']))
			{
				array_pop($_SESSION[$_SERVER['REMOTE_ADDR']][$arg_statusfile]['down']);
				$_SESSION[$_SERVER['REMOTE_ADDR']][$arg_statusfile]['down'] = array_merge(array($return['down']),$_SESSION[$_SERVER['REMOTE_ADDR']][$arg_statusfile]['down']);
				$return['down'] = @round((array_sum($_SESSION[$_SERVER['REMOTE_ADDR']][$arg_statusfile]['down'])/1024)/10,0);
			}
			elseif ($return['down'] !== 0)
			{
				$_SESSION[$_SERVER['REMOTE_ADDR']][$arg_statusfile]['down'] = array_fill(0,10,$return['down']);
			}
			
			/* upload speed */
			if (isset($_SESSION[$_SERVER['REMOTE_ADDR']][$arg_statusfile]['up']))
			{
				array_pop($_SESSION[$_SERVER['REMOTE_ADDR']][$arg_statusfile]['up']);
				$_SESSION[$_SERVER['REMOTE_ADDR']][$arg_statusfile]['upload'] = array_merge(array($return['up']),$_SESSION[$_SERVER['REMOTE_ADDR']][$arg_statusfile]['up']);
				$return['up'] = @round((array_sum($_SESSION[$_SERVER['REMOTE_ADDR']][$arg_statusfile]['up'])/1024)/10,0);
			}
			elseif ($return['up'] !== 0)
			{
				$_SESSION[$_SERVER['REMOTE_ADDR']][$arg_statusfile]['up'] = array_fill(0,10,$return['up']);
			}
		}
		else
		{
			unset($_SESSION[$_SERVER['REMOTE_ADDR']][$arg_statusfile]);
		}
		
		/* status */
		if (!empty($status['14']))
		{
			$return['status']	= $status['14'];
		}
		elseif ($return['percent'] == 100)
		{
			$return['status'] = "Seeding";
		}
		else
		{
			$data_left 			= ($status['11']-$status['9'])/1024;
			$return['status'] = @round($data_left/$return['down'],0);
		}

		return $return;
	}



	/* write to status file */
	function write_status($arg_torrentfile,$arg_pid,$arg_time,$arg_path,$arg_seeders,$arg_leechers,$arg_peers,$arg_pices_completed,$arg_pices_avaiable,$arg_pices_total,$arg_download_total,$arg_upload_total,$arg_torrent_size,$arg_download_speed,$arg_upload_speed,$arg_status)
	{
		/* 0: pid
		   1: time
		   2: path
		   3: seeders
		   4: leeachers
		   5: peers
		   6: pieces completed
		   7: pices available
		   8: pices in torrent
		   9: downloaded
		  10: uploaded
		  11: torrent size
		  12: download speed
		  13: upload speed
		  14: status (if any)
		*/

		$writedata = "$arg_pid\n$arg_time\n$arg_path\n$arg_seeders\n$arg_leechers\n$arg_peers\n$arg_pices_completed\n$arg_pices_avaiable\n$arg_pices_total\n$arg_download_total\n$arg_upload_total\n$arg_torrent_size\n$arg_download_speed\n$arg_upload_speed\n$arg_status\n";

		if (etc::php4_file_put_contents($arg_torrentfile.".status",$writedata))
		{
			return true;
		}
	}



	/* create a new torrent file */
	function new_status_file_by_torrentfile($arg_torrentfile)
	{
		if (!torrent::write_status($arg_torrentfile,0,0,"",0,0,0,0,0,0,0,0,torrent::size($arg_torrentfile),0,0,"New"))
		{
			return;
		}
		chmod($arg_torrentfile.".status",0600);
		return true;
	}



	/* stop torrent */
	function stop_by_torrentfile($arg_torrentfile)
	{
		$statusfile = $arg_torrentfile.".status";
		$status	    = torrent::status($statusfile);

		if (!$status['pid'] || !posix_kill($status['pid'],9))
		{
			return;
		}

		torrent::write_status($arg_torrentfile,0,0,$status['path'],0,0,0,0,0,0,$status['current_size'],0,$status['torrent_size'],0,0,"Stopped");
		w3btorrent::write2log("Stopped '".basename($arg_torrentfile)."'.");
	}



	/* start wctorrent */
	function start_wctorrent_by_torrentfile($arg_torrentfile,$arg_max_download_rate,$arg_max_upload_rate,$arg_min_peers,$arg_max_peers,$arg_port,$arg_save_dir = "",$arg_max_live_finish = "")
	{
		$statusfile = $arg_torrentfile.".status";
		$status	   = torrent::status($statusfile);
		/* make sure it's not running */
		if ($status['pid'] || !torrent::valid_file($arg_torrentfile))
		{
			return;
		}

		/* got valid settings? */
		if (!is_numeric($arg_max_download_rate.$arg_max_upload_rate.$arg_max_peers.$arg_min_peers.$arg_port.$arg_max_live_finish))
		{
			return;
		}

		/* let's be safe */
		$file = escapeshellarg($arg_torrentfile);
		$exec = $_SESSION[$_SERVER['REMOTE_ADDR']]['bin']['nohup']." ".$_SESSION[$_SERVER['REMOTE_ADDR']]['bin']['wctorrent']." $file -D $arg_max_download_rate -U $arg_max_upload_rate -m $arg_min_peers -M $arg_max_peers -p $arg_port";
		if (!empty($arg_save_dir))
		{
			$exec .= " -s ".escapeshellarg($arg_save_dir);
		}
		if ($arg_max_live_finish !== "")
		{ 
			$exec .= " -e $arg_max_live_finish";
		}
		$exec .= " >> ".escapeshellarg(realpath($_SESSION[$_SERVER['REMOTE_ADDR']]['sysdir']."/.htlog"))." &";
		etc::safe_exec($exec);
		w3btorrent::write2log("Started '".basename($arg_torrentfile)."' with `wctorrent`.");
	}
	
	
	/* start bt2status */
	function start_bt2status_by_torrentfile($arg_torrentfile,$arg_max_download_rate,$arg_max_upload_rate,$arg_min_peers,$arg_min_port,$arg_max_port,$arg_min_uploads,$arg_max_uploads,$arg_super_seeder,$arg_save_dir = "",$arg_max_live_finish = "")
	{
		$statusfile = $arg_torrentfile.".status";
		$status	   = torrent::status($statusfile);
		/* make sure it's not running */
		if ($status['pid'] || !torrent::valid_file($arg_torrentfile))
		{
			return;
		}

		/* got valid settings? */
		if (!is_numeric($arg_max_download_rate.$arg_max_upload_rate.$arg_min_peers.$arg_min_port.$arg_max_port.$arg_min_uploads.$arg_max_uploads.$arg_super_seeder.$arg_max_live_finish))
		{
			return;
		}

		/* let's be safe */
		$file		= escapeshellarg($arg_torrentfile);
		$ddir	= escapeshellarg($_SESSION[$_SERVER['REMOTE_ADDR']]['ddir']);
		
		/* here we need to set HOME so BitTorrent can create it's directory there
		   and also export TERM to trick it thinking we run it from a terminal */
		$exec	 	= "export HOME=$ddir;export TERM=\"xterm\";";
		$exec   .= $_SESSION[$_SERVER['REMOTE_ADDR']]['bin']['nohup']." ".$_SESSION[$_SERVER['REMOTE_ADDR']]['bin']['bt2status']." ".($arg_max_live_finish === ""?"forever":$arg_max_live_finish)." --responsefile $file --display_interval 5 --max_download_rate $arg_max_download_rate --max_upload_rate $arg_max_upload_rate --min_peers $arg_min_peers --min_uploads $arg_min_uploads --max_uploads $arg_max_uploads --minport $arg_min_port --maxport $arg_max_port --super_seeder $arg_super_seeder";
		
		/* where to save it */
		if (!empty($arg_save_dir))
		{
			$exec .= " --saveas ".escapeshellarg($arg_save_dir);
		}
		$exec .= " >> ".escapeshellarg(realpath($_SESSION[$_SERVER['REMOTE_ADDR']]['sysdir']."/.htlog"))." &";
		etc::safe_exec($exec);
		w3btorrent::write2log("Started '".basename($arg_torrentfile)."' with `bt2status`.");
	}



	/* delete torrent and it's status file */
	function delete_by_torrentfile($arg_torrentfile)
	{
		$statusfile = $arg_torrentfile.".status";
		$status	   = torrent::status($statusfile);
		/* make sure it's not running */
		if ($status['pid'])
		{
			return;
		}

		if (file_exists($arg_torrentfile))
		{
			unlink($arg_torrentfile);
			w3btorrent::write2log("Deleted '".basename($arg_torrentfile)."'.");
		}
		if (file_exists($statusfile))
		{
			unlink($statusfile);
			w3btorrent::write2log("Deleted '".basename($statusfile)."'.");
		}
	}
	
	
	
	/* stop all torrents by directory */
	function stop_all_by_dir($arg_dir)
	{
		foreach (hdc::list_files($arg_dir) as $filename)
		{
			$file = $arg_dir."/$filename";
			if (substr($file,-8) != ".torrent" || !is_file($file) || !is_readable($file))
			{
				continue;
			}
			torrent::stop_by_torrentfile($file);
		}
	}
}

?>
