#!/usr/bin/python

# Written by Bram Cohen
# see LICENSE.txt for license information

from BitTornado import PSYCO
if PSYCO.psyco:
    try:
        import psyco
        assert psyco.__version__ >= 0x010100f0
        psyco.full()
    except:
        pass
    
from BitTornado.download_bt1 import BT1Download, defaults, parse_params, get_usage, get_response
from BitTornado.RawServer import RawServer, UPnP_ERROR
from random import seed
from socket import error as socketerror
from BitTornado.bencode import bencode
from BitTornado.natpunch import UPnP_test
from threading import Event
from os.path import abspath
from sys import argv, stdout
import sys, os
from sha import sha
from time import strftime
from BitTornado.clock import clock
from BitTornado import createPeerID, version
from BitTornado.ConfigDir import ConfigDir

assert sys.version >= '2', "Install Python 2.0 or greater"
try:
    True
except:
    True = 1
    False = 0

PROFILER = False

def hours(n):
    if n == 0:
        return 'complete!'
    try:
        n = int(n)
        assert n >= 0 and n < 5184000  # 60 days
    except:
        return '<unknown>'
    m, s = divmod(n, 60)
    h, m = divmod(m, 60)
    if h > 0:
        return '%d hour %02d min %02d sec' % (h, m, s)
    else:
        return '%d min %02d sec' % (m, s)

class HeadlessDisplayer:
    def __init__(self):
        self.done = False
        self.file = ''
        self.percent = '';
        self.timeEst = ''
        self.downloadTo = ''
        self.downRate = ''
        self.upRate = ''
        self.errors = []
        self.last_update_time = -1
			# w3btorrent
        self.torrentfile		= '';
        self.pid 					= os.getpid();
        self.time 				= '';
        self.path					= '';
        self.seeders 			= '';
        self.leechers			= '';
        self.peers				= '';
        self.pieces_completed	= '';
        self.pieces_total		= '';
        self.pieces_available	= '';
        self.down_total			= '';
        self.up_total			= '';
        self.size					= '';
        self.finish_time		= -1;
        self.seed_time			= -1;
        
    def finished(self):
        if self.seed_time != -1:
    	     self.finish_time = (self.seed_time+clock());
        self.done = True;
        self.timeEst = 'Seeding';
        self.downRate = '';
        self.upRate	 = '';
        self.display();

    def failed(self):
        self.done = True;
        self.timeEst = 'Stopped';
        self.downRate = '';
        self.upRate	 = '';
        self.display();

    def error(self, errormsg):
        self.errors.append(errormsg)
        self.display()

    def display(self, dpflag = Event(), fractionDone = None, timeEst = None, 
            downRate = None, upRate = None, activity = None,
            statistics = None,  **kws):
        if self.last_update_time + 0.1 > clock() and fractionDone not in (0.0, 1.0) and activity is not None:
            return
        self.last_update_time = clock()
        if fractionDone is not None:
        		self.percent = str(float(int(fractionDone * 1000)) / 10)
        #if timeEst is not None:
        #    self.timeEst = hours(timeEst)
        #else:
        #		self.timeEst = 'Check exists';
        #if activity is not None and not self.done:
        #    self.timeEst = activity
        if downRate is not None:
            self.downRate = float(downRate);
        if upRate is not None:
            self.upRate = float(upRate);
        if statistics is not None:
           if not self.done:
              self.seeders = statistics.numSeeds;
           else:
              self.seeders = statistics.numOldSeeds;
           self.peers 				= statistics.numPeers;
           self.leechers			= (self.peers - self.seeders);
           self.pieces_completed	= statistics.storage_numcomplete;
           self.pieces_available	= (statistics.numCopies*1000);
           self.pieces_total		= statistics.storage_totalpieces;
           self.down_total			= statistics.downTotal;
           self.up_total			= statistics.upTotal;
           
           
        #print '\n\n\n\n'
        #for err in self.errors:
        #    print 'ERROR:\n' + err + '\n'
        #print 'saving:        ', self.file
        #print 'percent done:  ', self.percentDone
        #print 'time left:     ', self.timeEst
        #print 'download to:   ', self.downloadTo
        #print 'download rate: ', self.downRate
        #print 'upload rate:   ', self.upRate
        #print 'share rating:  ', self.shareRating
        #print 'seed status:   ', self.seedStatus
        #print 'peer status:   ', self.peerStatus
        #stdout.flush()
        self.write_status();
        dpflag.set()        
        

    def write_status(self):
    	if self.finish_time != -1 and self.finish_time < clock():
    		self.timeEst = 'Finish';
    	fd=open(self.torrentfile+'.status','w');
    	
    	fd.write(str(self.pid)+'\n');
    	fd.write(str(clock())+'\n');
    	fd.write(str(self.downloadTo)+'\n');
    	fd.write(str(self.seeders)+'\n');
    	fd.write(str(self.leechers)+'\n');
    	fd.write(str(self.peers)+'\n');
    	fd.write(str(self.pieces_completed)+'\n');
    	fd.write(str(self.pieces_available)+'\n');
    	fd.write(str(self.pieces_total)+'\n');
    	fd.write(str(self.down_total)+'\n');
    	fd.write(str(self.up_total)+'\n');
    	fd.write(str(self.size)+'\n');
    	fd.write(str(self.downRate)+'\n');
    	fd.write(str(self.upRate)+'\n');
    	fd.write(str(self.timeEst)+'\n');
    	for error in self.errors:
    		fd.write(str(error)+'\n');
    	fd.close;
    	
    	if self.timeEst == 'Finish':
    		raise KeyboardInterrupt; # don't know a better way
      

    def chooseFile(self, default, size, saveas, dir):
    	self.size	= size;
        self.file = '%s (%.1f MB)' % (default, float(size) / (1 << 20))
        if saveas != '':
            default = saveas
        self.downloadTo = abspath(default)
        return default

    def newpath(self, path):
        self.downloadTo = path

def run(arg_seed_time,params):
    try:
        import curses
        curses.initscr()
        cols = curses.COLS
        curses.endwin()
    except:
        cols = 80

    h = HeadlessDisplayer()
    while 1:
        configdir = ConfigDir('downloadheadless')
        defaultsToIgnore = ['responsefile', 'url', 'priority']
        configdir.setDefaults(defaults,defaultsToIgnore)
        configdefaults = configdir.loadConfig()
        defaults.append(('save_options',0,
         "whether to save the current options as the new default configuration " +
         "(only for btdownloadheadless.py)"))
        try:
            config = parse_params(params, configdefaults)
        except ValueError, e:
            print 'error: ' + str(e) + '\nrun with no args for parameter explanations'
            break
        if not config:
            print get_usage(defaults, 80, configdefaults)
            break
        if config['save_options']:
            configdir.saveConfig(config)
        configdir.deleteOldCacheData(config['expire_cache_data'])

        myid = createPeerID()
        seed(myid)
        
        doneflag = Event()
        def disp_exception(text):
            print text
        rawserver = RawServer(doneflag, config['timeout_check_interval'],
                              config['timeout'], ipv6_enable = config['ipv6_enabled'],
                              failfunc = h.failed, errorfunc = disp_exception)
        upnp_type = UPnP_test(config['upnp_nat_access'])
        while True:
            try:
                listen_port = rawserver.find_and_bind(config['minport'], config['maxport'],
                                config['bind'], ipv6_socket_style = config['ipv6_binds_v4'],
                                upnp = upnp_type, randomizer = config['random_port'])
                break
            except socketerror, e:
                if upnp_type and e == UPnP_ERROR:
                    print 'WARNING: COULD NOT FORWARD VIA UPnP'
                    upnp_type = 0
                    continue
                print "error: Couldn't listen - " + str(e)
                h.failed()
                return

        response = get_response(config['responsefile'], config['url'], h.error)
        if not response:
            break

        # w3btorrent
        try:
        		t = int(arg_seed_time);
        		h.seed_time = (int(arg_seed_time)*3600);
        except:
        		print 'argument 2 must be numeric';
        		break;
        h.torrentfile 	= config['responsefile'];
        
        infohash = sha(bencode(response['info'])).digest()

        dow = BT1Download(h.display, h.finished, h.error, disp_exception, doneflag,
                        config, response, infohash, myid, rawserver, listen_port,
                        configdir)

        if not dow.saveAs(h.chooseFile, h.newpath):
            break
        if not dow.initFiles(old_style = True):
            break
        if not dow.startEngine():
            dow.shutdown()
            break
        dow.startRerequester()
        dow.autoStats()

        if not dow.am_I_finished():
            h.display(timeEst = 'Connecting')
        rawserver.listen_forever(dow.getPortHandler())
        h.display(timeEst = 'Stopping')
        dow.shutdown()
        break
    try:
        rawserver.shutdown()
    except:
        pass
    if not h.done:
        h.failed()

if __name__ == '__main__':
    if argv[1:] == ['--version']:
        print version
        sys.exit(0)

    if PROFILER:
        import profile, pstats
        p = profile.Profile()
        p.runcall(run,argv[1:])
        log = open('profile_data.'+strftime('%y%m%d%H%M%S')+'.txt','a')
        normalstdout = sys.stdout
        sys.stdout = log
#        pstats.Stats(p).strip_dirs().sort_stats('cumulative').print_stats()
        pstats.Stats(p).strip_dirs().sort_stats('time').print_stats()
        sys.stdout = normalstdout
    else:
	run(argv[1],argv[2:]);
